"""
This file is a data fixture for test_test.py
it is supposed to be discovered and loaded as
as test module in recursive search for test files.
This file is not meant to be imported, nor to implement
any feature.
"""
