"""
distutilazy.command.clean_pyc
-----------------------------

Command to clean compiled python files

:license: MIT, see LICENSE for more details.
"""

import distutilazy.clean


class clean_pyc(distutilazy.clean.CleanPyc):
    pass