"""
distutilazy.command.clean_all
-----------------------------

Command to clean all temporary files

:license: MIT, see LICENSE for more details.
"""

import distutilazy.pyinstaller


class clean_all(distutilazy.pyinstaller.CleanAll):
    pass
